package cz.zcu.mjaneck.bak_prace.app;


import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import javafx.collections.ObservableMap;
import javafx.embed.swing.SwingFXUtils;
import javafx.scene.chart.XYChart;
import javafx.scene.image.WritableImage;
import cz.zcu.mjaneck.bak_prace.data.ChartData;
import cz.zcu.mjaneck.bak_prace.data.Measurement;

import javax.imageio.ImageIO;
import java.io.*;
import java.util.*;
import java.util.stream.Collectors;

/**
 * Tato třída slouží pro práci s daty. Ukládá a načítá data z disku.
 */
public class DataHandler {
    /** Instance třídy */
    private static DataHandler INSTANCE;
    /** Proměnná obsahující všechny programem používané řetězce*/
    private Properties properties;
    /** Mapa kapacit měření*/
    private ObservableMap<String, Double> capacityMap;

    /**
     * Privátní konstruktor - třída je implementována jako singleton
     */
    private DataHandler() {
        // skutečně prázdný
    }

    /**
     * Vrací instanci třídy
     * @return - instance třídy
     */
    public static DataHandler getInstance() {

        if (INSTANCE == null)
            INSTANCE = new DataHandler();

        return INSTANCE;
    }

    /**
     * Setter pro řetězcový zdroj
     * @param properties - zdroj řetězců
     */
    public void setProperties(Properties properties) {
        this.properties = properties;
    }

    /**
     * Uloží obraz grafu na disk
     * @param image - obraz grafu
     * @param imageName - název obrazu
     * @return - true pokud metoda proběhla úspěšně
     */
    public boolean exportChart(WritableImage image, String imageName) {
        File file = new File(System.getProperty("user.dir")
                + properties.getProperty("image.folder")
                + imageName
                + properties.getProperty("image.extension"));

        if (!file.getParentFile().mkdirs()) {
            if (!file.getParentFile().exists()) {
                return false;
            }
        }

        try {
            ImageIO.write(SwingFXUtils.fromFXImage(image, null), properties.getProperty("image.format"), file);
        } catch (IOException e) {
            return false;
        }

        return true;
    }

    /**
     * Uloží data měření na disk
     * @param file - soubor do kterého budou data uložena
     * @param series - data měření
     * @return - true pokud metoda proběhla úspěšně
     */
    public boolean exportData(File file, XYChart.Series<Number, Number> series) {
        String dataJson;
        Measurement chartData = new Measurement();

        for (XYChart.Data<Number, Number> data : series.getData()) {
            chartData.getData().add(new ChartData(data.getXValue(), data.getYValue()));
        }

        chartData.setCapacity(capacityMap.get(series.getName().substring(0, series.getName().lastIndexOf('-'))));

        try {
            dataJson = new ObjectMapper().writeValueAsString(chartData);
            BufferedWriter bw = new BufferedWriter(new FileWriter(file));
            bw.write(dataJson);
            bw.close();
        } catch (IOException e) {
            return false;
        }

        return true;
    }

    /**
     * Načítá data z disku
     * @param file - soubor který bude načten
     * @return - pole s naměřenými hodnotami, null pokud nastala chyba
     */
    public XYChart.Series<Number, Number> loadData(File file) {
        BufferedReader br;
        ObjectMapper mapper = new ObjectMapper();

        try {
            br = new BufferedReader(new FileReader(file));
        } catch (FileNotFoundException e) {
            return null;
        }

        Measurement data;

        try {
            String dataJson = br.lines().collect(Collectors.joining());
            data = mapper.readValue(dataJson, Measurement.class);
        } catch (JsonProcessingException | NullPointerException e) {
            return null;
        }

        String name = file.getName().substring(0, file.getName().lastIndexOf("."));
        XYChart.Series<Number, Number> series = new XYChart.Series<>();
        series.setName(name);

        for (ChartData chartData : data.getData()) {
            series.getData().add(new XYChart.Data<>(chartData.getxData(), chartData.getyData()));
        }

        String key = name.substring(0, name.lastIndexOf('-'));

        if (capacityMap.containsKey(key)) {
            return null;
        } else {
            capacityMap.put(key, data.getCapacity());
            return series;
        }
    }

    /**
     * Setter pro mapu kapacit
     * @param capacityMap - mapa kapacit
     */
    public void setCapacityMap(ObservableMap<String, Double> capacityMap) {
        this.capacityMap = capacityMap;
    }
}
